****** Make Friends with SAS, Day 4: Exporting SAS Output and using Analysis PROCs
****** Created by Lesa Hoffman;

* Output options: all can be turned on/off by adding or removing the 'NO':
  page number, date, centering, or page breaks, page length and width; 
OPTIONS nonumber nodate nocenter formdlim=' ' pagesize=MAX linesize=120;
TITLE; ODS TRACE OFF;  * Print names of output tables in log ON/OFF;
	
* Location of original data files;
%LET filepath=F:\13_Psyc930SAS\Day4_Output_Analysis_PROCs;
LIBNAME filepath "&filepath.";
	
* Let's read in data;
DATA work.saspractice; SET filepath.saspractice;
LABEL 	ID= 		"ID: Participant ID"
		female= 	"female: 0=M, 1=F"
		group=		"group: 0=Control, 1=Treatment"
		cognition=	"cog: Cognitive Score"
		affect=		"aff: Affect Score"
		behavior=	"beh: Behavior Score";
RUN;
DATA work.factordata; SET filepath.factordata;
RUN;


/* Common commands that can be used in any PROC:
   On PROC statement:
		NOPRINT 	suppresses all output (but still saves to files)
		NAMELEN=100	gives length of effects in output tables
	Other statements:
		BY			ask for output separately BY group
		WHERE		ask for output filtering with WHERE group
*/

*******************************************************************************
******* 				DATA DESCRIPTIVE STATISTICS						*******
******************************************************************************;

* Creating value labels for categorical variables;
* "Formats" are not stored with the data, but can be used in output;
* This creates 2 formats called "gender" and "agegroup" for later use;

PROC FORMAT;
	VALUE FGender 1 = "Male" 2 = "Female";
	VALUE FAgeGroup 1 = "Younger" 2 = "Older";
RUN;

* PROC FREQ gets frequencies and cross-tabulations for categorical variables;
* TABLES lists variables to be tabulated or cross-tabulated (indicated with *);
* CHISQ asks for chi-square test of significance -- many more tests available;
* EXACT asks for exact tests in addition to regular versions;
* FORMAT says apply "varformat" to var1 and var2;
* Add NOCOL or NOROW after / on TABLES to suppress the column and row percentages;

PROC FREQ DATA=work.datafilename;
	TABLES var1*var2 / CHISQ;
	EXACT CHISQ;
	FORMAT var1 varformat1. var2 varformat2.;
RUN;

* PROC MEANS gets descriptive statistics for continuous variables;
* PRINT/NOPRINT specifies whether you want output;
* Keywords ask for different summary statistics;
* CLASS asks for separate output per level of categorical variable;
* VAR lists variables to be included;
PROC MEANS PRINT DATA=work.datafilename
    	N NMISS MIN MAX RANGE SUM MEAN MEDIAN STDDEV;
	CLASS catvar1 catvar2;
	VAR var1 var2 var3 var4;
RUN;

* PROC UNIVARIATE also does additional summary statistics and LOTS of plots; 
PROC UNIVARIATE DATA=work.datafilename
    	ALL 		/* Request ALL stats and tables */
    	ALPHA=.10 	/* Specify confidence level for confidence limits */
    	CIBASIC 	/* CI for mean, SD, and variance */
    	NORMAL 		/* Request tests for normality */
    	PLOT; 		/* Stem and leaf, box, and normal plots */
    CLASS var1; 			/* Output per level of CLASS variables */
	VAR var1 var2; 			/* Variables to be included */
    HISTOGRAM var1; 		/* Histogram for specified variables */
RUN;


***** Now you try, using the work.saspractice dataset; 

* Create formats to use with female and group in the output;
PROC FORMAT;
	VALUE Fgender 0 = "Male" 1 = "Female";
	VALUE Fgroup  0 = "Control" 1 = "Treatment";
RUN;


* Send all of the following to a word document via ODS RTF, and title each:
  Get frequencies for female, group, and female*group 
  Get descriptive statistics for these variables by group: cognition affect behavior;

* Now try the other methods -- save as: HTML, XLS; 

* Send output to word document;
ODS RTF FILE="&filepath.\Descriptives.rtf"  STYLE=STATDOC;
* Built-in styles include default, minimal, nofontdefault, printer, rtf,
	statdoc, theme, fancyprinter, sansprinter, sasdocprinter, serifprinter;

* Save as HTML version instead;
ODS HTML FILE="&filepath.\Descriptives.html"  STYLE=HTMLBLUE;

* Save as excel version instead;
ODS HTML FILE="&filepath.\Descriptives.xls"  STYLE=MINIMAL;
	
* Get frequencies for female, group, and female*group -- title your ouptut;
TITLE1 "Frequencies for female, group, and female by group";
PROC FREQ DATA=work.saspractice;
	TABLES female group female*group / CHISQ NOCOL NOROW;
	EXACT CHISQ;
	FORMAT female Fgender. group Fgroup.;
RUN; TITLE1;

* Get descriptive statistics for these variables by group: cognition affect behavior;
TITLE1 "PROC MEANS for cognition, affect, and behavior";
PROC MEANS PRINT DATA=work.saspractice 
    	N NMISS MIN MAX RANGE SUM MEAN MEDIAN STDDEV;
	CLASS group; FORMAT group Fgroup.;
	VAR cognition affect behavior;
RUN; TITLE1;

* Close word file;  ODS RTF  CLOSE;
* Close HTML file;  ODS HTML CLOSE;
* Close excel file; ODS HTML CLOSE;

* Save a combined log+output text file; 
PROC PRINTTO PRINT="&filepath.\Descriptives.txt" 
             LOG=  "&filepath.\Descriptives.txt"; RUN;

* Get frequencies for female, group, and female*group -- title your ouptut;
TITLE1 "Frequencies for female, group, and female by group";
PROC FREQ DATA=work.saspractice;
	TABLES female group female*group / CHISQ NOCOL NOROW;
	EXACT CHISQ;
	FORMAT female Fgender. group Fgroup.;
RUN; TITLE1;

* Get descriptive statistics for these variables by group: cognition affect behavior;
TITLE1 "PROC MEANS for cognition, affect, and behavior";
PROC MEANS PRINT DATA=work.saspractice 
    	N NMISS MIN MAX RANGE SUM MEAN MEDIAN STDDEV;
	CLASS group;
	VAR cognition affect behavior;
RUN; TITLE1;

* Close combined text file;  PROC PRINTTO; RUN;



*******************************************************************************
****** 			Correlation, Covariance, and Alpha						 ******
******************************************************************************;
	
* Correlations provided by default, keywords "ALPHA" and "COV" add those;	
TITLE1 "Cronbach's Alpha";
PROC CORR DATA=work.datafilename ALPHA NOMISS COV;
	VAR var1-var20;
RUN; TITLE1;

* Now you try, using the "factordata", send results to excel
  Get the correlation matrix, covariance matrix, and alpha for the items i1-i10;

* Send output to excel;
ODS HTML FILE="&filepath.\Item Analysis.xls" STYLE=statdoc;

* Get alpha for the items i1-i10;
TITLE1 "Cronbach's Alpha";
PROC CORR DATA=work.factordata ALPHA NOMISS COV;
	VAR i1-i10;
RUN; TITLE1;

* Close excel file; ODS HTML CLOSE;


TITLE "More Example Correlation Analysis";
PROC CORR DATA=work.datafilename
        PEARSON SPEARMAN		/* Request Pearson and Spearman r */
        COV						/* Request covariances */
        NOMISS NOSIMPLE;		/* Listwise deletion, suppress descriptives */
    PARTIAL controlvar;			/* Control for these variables */
    VAR var1 var2;				/* Variables in top of columns in order */
    WITH var3 var4;				/* Variables in rows in order */
RUN;

TITLE1 "Bare-Bones Correlation Analysis";
TITLE2 "For Everybody at Once";
PROC CORR DATA=work.saspractice;
	VAR cog aff beh;
	ODS OUTPUT PearsonCorr=corrsave SimpleStats=work.simplesave;
RUN; TITLE2;
* Save output to excel as separate worksheets per table;
PROC EXPORT DATA=corrsave OUTFILE= "&filepath.\&filename..xls" 
	DBMS=EXCEL REPLACE; SHEET= "Correlations for All"; RUN;
PROC EXPORT DATA=simplesave OUTFILE= "&filepath.\&filename..xls" 
	DBMS=EXCEL REPLACE; SHEET= "Simple Statistics"; RUN;


*** Now lets get correlations by group and save/export that output;

TITLE2 "Correlations by Group Instead";
* Must sort by BY variable(s) first;
PROC SORT DATA=work.saspractice; 
	BY group; 
RUN;
PROC CORR DATA=work.saspractice; 
	BY group;
	VAR cognition affect behavior;
	ODS OUTPUT PearsonCorr=corrsavebygroup SimpleStats=work.simplesavebygroup;
RUN; TITLE1; TITLE2;
* Save output to excel;
PROC EXPORT DATA=corrsavebygroup OUTFILE= "&filepath.\Example.xls" 
	DBMS=EXCEL REPLACE; SHEET= "Correlations by Group"; RUN;
PROC EXPORT DATA=simplesavebygroup OUTFILE= "&filepath.\Example.xls" 
	DBMS=EXCEL REPLACE; SHEET="Simple Stats by Group"; RUN;



*******************************************************************************
****** 			General Linear Model Analyses via PROC REG				 ******
******************************************************************************;

TITLE1 "Example Regression Analysis";
PROC REG DATA=work.datafilename SIMPLE;	/* Prints simple descriptive stats */
    MODEL dv = iv1 iv2 			/* Predict dv(s) from iv1 and iv2  */
		/ ALL					/* Print ALL statistics */
		  STB					/* Standardized regression coefficients */
		  TOL VIF				/* Prints tolerance, variance inflation factors  */
          SELECTION= ;			/* Enter predictors: forward, backward, stepwise */
    PLOT dv*iv1=idvar;			/* Scatterplot of dv & iv5, points labeled by idvar */
    PRINT ANOVA;				/* Produce ANOVA of regression model */
	OUTPUT OUT=work.savedata	/* Dataset for saved info below */
		PREDICTED=pred			/* Name predicted values with variable "pred" */
		RESIDUAL=resid;			/* Name residual values with variable "resid" */
	label: TEST [constraints];  /* For custom hypothesis tests */
RUN; QUIT;						/* You need run and quit because can have multiple models */
* See PROC REG documentation for lots more info you can request;

TITLE2 "Example Regression by Group";
PROC REG DATA=work.saspractice;
	BY group; FORMAT Fgroup.;
	MODEL behavior = cognition        / STB TOL VIF;
	MODEL behavrio = cognition affect / STB TOL VIF;
	* Example requested contrast -- are the regression coefficients equal?;
	Equal_contributors: TEST cognition = affect;
	ODS OUTPUT  ParameterEstimates=work.parms FitStatistics=work.fit
				ANOVA=work.anova;
RUN; QUIT; TITLE1; TITLE2;
PROC EXPORT DATA=parms OUTFILE= "&filepath.\Regression.xls" 
	DBMS=EXCEL REPLACE; SHEET= "Parameters"; RUN;
PROC EXPORT DATA=fit OUTFILE= "&filepath.\Regression.xls" 
	DBMS=EXCEL REPLACE; SHEET= "Fit Statistics"; RUN;
PROC EXPORT DATA=ANOVA OUTFILE= "&filepath.\Regression.xls" 
	DBMS=EXCEL REPLACE; SHEET= "ANOVA"; RUN;



*******************************************************************************
****** 			General Linear Model Analyses via PROC GLM				 ******
******************************************************************************;

* NOTE: PROC GLM can be used to do regression or ANOVA, and doesn't require
  pre-defined interaction terms, which makes it more convenient than PROC REG;

* The only thing it doesnt do is R2-change F-tests, which I wrote a macro program for;

TITLE1 "Example ANOVA Analysis";
PROC GLM DATA=work.datafilename;
    CLASS catvar1 catvar2;						  	/* Dummy-codes categorical variables, 0=highest */
    MODEL dv = catvar1 catvar2 catvar1*catvar2    	/* Predict dv from catvar1, catvar2, & interaction */ 
		/ SOLUTION								  	/* Prints regression solution */
          NOINT NOUNI;							  	/* No intercept, suppress univariate tests if manova */
    CONTRAST 
		"catvar1 g2 vs. g3" catvar1 0 -.5 .5; 	   	/* Label and test specific hypotheses */          
    MEANS catvar1 catvar2 catvar1*catvar2 		   	/* Means, SD, for each effect */
		/ TUKEY; 									/* Type1 error correction for followups */
    LSMEANS catvar1*catvar2							/* Predicted means for unbalanced designs */
		/ AT (var1) = (0)							/* Predicted means at specified level of covariate */
		  PDIFF=ALL									/* All possible follow-up tests */
		  SLICE=catvar2;							/* Tests effect of catvar1 per level of catvar2 */
													/* Variables in means or lsmeans must be in class statement */
	ESTIMATE "label" [contrasts];					/* For custom hypothesis tests */
RUN; QUIT;  TITLE1;									/* Multiple models are not allowed, but it still wants QUIT */

TITLE1 "Descriptives for Affect and Cognition";
PROC MEANS DATA=work.saspractice; 
	VAR affect cognition;
RUN; TITLE1;

TITLE1 "Our Example Regression with Interaction Term";
TITLE2 "Affect Z-Score by Cognition Z-Score";
TITLE3 " | @2 fits all main effects and up to 2-way interactions";
PROC GLM DATA=work.saspractice;
	MODEL behavior = affect|cognition@2 / SS3 SOLUTION;
* To get any kind of regression slope, use ESTIMATE;
	ESTIMATE "Affect Slope for Cognition= -1"	affect 1 affect*cognition -1;
	ESTIMATE "Affect Slope for Cognition=  0"	affect 1 affect*cognition  0 ;
	ESTIMATE "Affect Slope for Cognition=  1"	affect 1 affect*cognition  1;
	ESTIMATE "Cognition Slope for Affect= -1"	cognition 1 affect*cognition -1;
	ESTIMATE "Cognition Slope for Affect=  0"	cognition 1 affect*cognition  0 ;
	ESTIMATE "Cognition Slope for Affect=  1"	cognition 1 affect*cognition  1;
RUN; QUIT; ODS HTML CLOSE; ODS HTML; TITLE1; TITLE2; TITLE3;


* Cals example from GLM Module;
PROC IMPORT DATAFILE="&filepath.\Cal_2xK_GLM.sav" OUT=work.CalGLM DBMS=SAV REPLACE;
RUN;
TITLE1 "Descriptives for #Practices";
PROC MEANS DATA=work.CalGLM; 
	VAR numpract;
RUN; TITLE1;

TITLE1 "Example 2-way Interaction ANOVA (2x3)";
TITLE2 "Practice Group (Easier, Harder, Same) by Review Attendance (No, Yes)";
TITLE3 " | @2 fits all main effects and up to 2-way interactions";
TITLE4 "Categorical Predictors are in order of Value Labels (FORMATS)";
PROC GLM DATA=work.CalGLM;
	CLASS practgrp atndrev;	 * CLASS indicates which predictors are categorical;
	MODEL testperf = practgrp|atndrev@2 / SS3; 
* To get any kind of mean, mean difference, or t/F test, use LSMEANS;
	LSMEANS practgrp atndrev 			/* Get all marginal means  */
		/ LINES TDIFF STDERR PDIFF=ALL;   	/* Get all t-tests for marginal mean diffs */
	LSMEANS practgrp*atndrev 			/* Get all cell means */
		/ LINES TDIFF STDERR PDIFF=ALL		/* Get all t-tests for cell mean diffs */
		  SLICE=practgrp SLICE=atndrev; /* Get simple main effect F-tests BY slice */
RUN; QUIT; ODS HTML CLOSE; ODS HTML; TITLE1; TITLE2; TITLE3; TITLE4;


TITLE1 "Example 2-way Interaction (3xContinuous)";
TITLE2 "Practice Group (Easier, Harder, Same) by #Practices";
TITLE3 " | @2 fits all main effects and up to 2-way interactions";
TITLE4 "Categorical Predictors are in order of Value Labels (FORMATS)";
PROC GLM DATA=work.CalGLM;
	CLASS practgrp;		* Continuous predictor is not on CLASS;
	MODEL testperf = practgrp|numpract@2 / SS3; 
* To get any kind of mean, mean difference, or t/F test, use LSMEANS;
	LSMEANS practgrp  				 /* Get marginal means AT specific #practices  */
		/ AT (numpract) = (3) 
		  TDIFF STDERR PDIFF=ALL;   	/* Get t-tests for all marginal mean diffs */
	LSMEANS practgrp  				 /* Get marginal means AT specific #practices  */
		/ AT (numpract) = (6) 
		  TDIFF STDERR PDIFF=ALL;   	/* Get t-tests for all marginal mean diffs */
	LSMEANS practgrp  				 /* Get marginal means AT specific #practices  */
		/ AT (numpract) = (9) 
		  TDIFF STDERR PDIFF=ALL;   	/* Get t-tests for all marginal mean diffs */
* To get any kind of regression slope, use ESTIMATE;
	ESTIMATE "#Practices Slope for Easy   Group"	numpract 1 practgrp*numpract 1 0 0;
	ESTIMATE "#Practices Slope for Harder Group"	numpract 1 practgrp*numpract 0 1 0;
	ESTIMATE "#Practices Slope for Same   Group"	numpract 1 practgrp*numpract 0 0 1;
RUN; QUIT; ODS HTML CLOSE; ODS HTML; TITLE1; TITLE2; TITLE3; TITLE4;


* Now you try: in work.saspractice, predict cognition, affect, and behavior from female*group
   Get all simple effects, marginal means, and cell means;

* HINT: there are two ways to automate this process
	(1) stack the DVs into one column, and use BY
	(2) use a macro program to automate this process;


* (1) Stack by DV, and use BY;
DATA work.stacked; SET work.saspractice;
	DV="1cognition"; outcome=cognition; OUTPUT;
	DV="2affect"; 	 outcome=affect;    OUTPUT;
	DV="3behavior";  outcome=behavior;  OUTPUT;
RUN;
PROC SORT DATA=work.stacked;
	BY DV;
RUN;
TITLE1 "Example 2-way Interaction ANOVA (2x2)";
TITLE2 "Female by Group";
TITLE3 " | @2 fits all main effects and up to 2-way interactions";
TITLE4 "Categorical Predictors are in order of Value Labels (FORMATS)";
PROC GLM DATA=work.stacked;
	BY DV;
	CLASS female group;	 * CLASS indicates which predictors are categorical;
	FORMAT female Fgender. group Fgroup.;
	MODEL outcome = female|group@2 / SS3; 
* To get any kind of mean, mean difference, or t/F test, use LSMEANS;
	LSMEANS female group 			/* Get all marginal means  */
		/ TDIFF STDERR PDIFF=ALL;   	/* Get all t-tests for marginal mean diffs */
	LSMEANS female*group 			/* Get all cell means */
		/  TDIFF STDERR PDIFF=ALL		/* Get all t-tests for cell mean diffs */
		  SLICE=female SLICE=group; /* Get simple main effect F-tests BY slice */
RUN; QUIT; ODS HTML CLOSE; ODS HTML; TITLE1; TITLE2; TITLE3; TITLE4;


* (2) Use a macro program;
%MACRO RunByDV(DV);
TITLE1 "Example 2-way Interaction ANOVA (2x2)";
TITLE2 "Female by Group";
TITLE3 " | @2 fits all main effects and up to 2-way interactions";
TITLE4 "Categorical Predictors are in order of Value Labels (FORMATS)";
PROC GLM DATA=work.saspractice;
	CLASS female group;	 * CLASS indicates which predictors are categorical;
	FORMAT female Fgender. group Fgroup.;
	MODEL &DV. = female|group@2 / SS3; 
* To get any kind of mean, mean difference, or t/F test, use LSMEANS;
	LSMEANS female group 			/* Get all marginal means  */
		/ TDIFF STDERR PDIFF=ALL;   	/* Get all t-tests for marginal mean diffs */
	LSMEANS female*group 			/* Get all cell means */
		/  TDIFF STDERR PDIFF=ALL		/* Get all t-tests for cell mean diffs */
		  SLICE=female SLICE=group; /* Get simple main effect F-tests BY slice */
RUN; QUIT; ODS HTML CLOSE; ODS HTML; TITLE1; TITLE2; TITLE3; TITLE4;
%MEND RunByDV;
%RunByDV(cognition);
%RunByDV(affect);
%RunByDV(behavior);




*******************************************************************************
****** 					Exploratory Factor Analysis						 ******
******************************************************************************;

TITLE1 "Cronbach's Alpha";
PROC CORR DATA=work.datafilename ALPHA NOMISS;
	VAR var1-var20;
RUN; TITLE1;

TITLE1 "Example Exploratory Factor Analysis";
PROC FACTOR DATA=work.datafilename
		SCREE					/* Print scree plot */
		REORDER					/* Order variables in order of contribution */
		NFACTORS=				/* Specify number of factors */
		MINEIGEN=				/* Specify minimum eigenvalues */
		METHOD=					/* Extraction method: */
			ML						/* Maximum likelihood */
			PRIN					/* Principal components if priors = 1 */
			PRIN					/* Principal factors if priors NE 1 */
			PRINIT					/* Iterative principal factors */
		PRIORS=					/* Starting communality estimates */
			ONE						/* All variances on diagonal --> PCA */
			SMC						/* Squared multiple correlations --> EFA */				
		ROTATE=					/* Rotation method */
			VARIMAX QUARTIMAX EQUIMAX		/* Orthogonal */
			PROMAX;							/* Oblique (still gives varimax, too) */
	VAR var1-var20;
RUN; TITLE1;

TITLE1 "Bare-Bones Principal Component Analysis";
TITLE2 "Eigenvalues over 1, Varimax Rotation";
PROC FACTOR DATA=work.datafilename
		MINEIGEN=1 METHOD=PRIN PRIORS=ONE ROTATE=VARIMAX;
	VAR var1-var20;
RUN; TITLE1; TITLE2;

TITLE1 "Bare-Bones Exploratory Factor Analysis";
TITLE2 "Eigenvalues over 1, ML Extraction, Promax Rotation";
PROC FACTOR DATA=work.datafilename
		MINEIGEN=1 METHOD=ML PRIORS=SMC ROTATE=PROMAX;
	VAR var1-var20;
RUN; TITLE1; TITLE2;


*******************************************************************************
****** 			MANOVA/REPEATED MEASURES VIA GLM						 ******
******************************************************************************;

* Note: PROC MIXED is a better way to do this using maximum likelihood, 
  but here is the least squares version;

TITLE1 "Example MANOVA/Repeated Measures Analysis";
PROC GLM DATA=work.datafilename;
    CLASS catvar1 catvar2;						/* Dummy-codes categorical variables, 0=highest */
    MODEL dv1 dv2 dv3 dv4 						/* List outcome variables (4) */
		  = catvar1 catvar2 catvar1*catvar2		/* 3 predictors (2 main effects, 1 interaction) */
    	/ SOLUTION; 							/* Print solution */
    MANOVA h=_all_; 							/* Print all tests */
    REPEATED dv 4 								/* 4 levels of outcome "DV" */
        CONTRAST 								/* Dummy-coded contrasts */
        POLYNOMIAL 								/* Generate polynomial contrasts */
        PROFILE 								/* Contrasts adjacent factor levels */
        HELMERT 								/* Each level vs. mean of subsequent levels */
        MEAN 									/* Each level vs. mean of all others */
        / SUMMARY;								/* ANOVA table for each contrast defined within-subject */
RUN; QUIT; TITLE1;
